function [ Zl, Zv, compFugIl, compFugIv,compFugCoIl,compFugCoIv] = CAP_PR_EOS(NCOMP,xi,yi, T, Pl, Pv,Tc, Pc, w, kijmat)

%Calculate reduced temperatures and pressures
Tr = T./Tc;
Pvr = Pv./Pc;
Plr = Pl./Pc;
R = 0.0083144; %MPa.m^3/mol.K 

%Calculate parameters for the Peng-Robinson EOS for pure fluids
ac = 0.45724 .*(R.*Tc).^2./Pc; %Equation 7.16
b  = 0.07780*R.*Tc./Pc; %Equation 7.16
for i=1:NCOMP
    if w(i) <= 0.49
m(i) = 0.3796 + 1.54226*w(i)- 0.2699*w(i)^2; %5113
% m = 0.3796 + 1.54226?  0.2699?2.
    else
m(i) = 0.379642 + 1.48503*w(i) - 0.1644*w(i)^2 + 0.016667*w(i)^3;
    end
end

alpha = (1+m.*(1-sqrt(Tr))).^2;%Equation 7.17
ac_alpha = ac.*alpha;

%%%%%%%%%%%%%%%%%%%% 
%       Liquid     %
%%%%%%%%%%%%%%%%%%%%
ac_Mixl=0;
for j = 1:NCOMP
    for i = 1:1:NCOMP
       ac_Mixl = ac_Mixl+ xi(i)*xi(j)* sqrt(ac_alpha(i)*ac_alpha(j))*(1-kijmat(i,j));
    end
end


Al = ac_Mixl*Pl/(R*T)^2; %Equation 7.21
%Calculate the B of mixing using formula on page 585
b_Mixl = sum(xi .* b);
Bl = b_Mixl*Pl/(R*T); %Equation 7.22

Psil=zeros(1,NCOMP)  ; %595
for i = 1:NCOMP
    Psil(i)=0;
    for j = 1:1:NCOMP
       Psil(i) = Psil(i)+ xi(j)* sqrt(ac_alpha(i)*ac_alpha(j))*(1-kijmat(i,j));
    end
end
%Calculate coefficients for the cubic equation to solve
a2l = -(1 - Bl);
a1l = Al-3*Bl^2-2*Bl;
a0l = -(Bl*Al-Bl^2-Bl^3);
%Store the coefficients in an array
polyCol = [1 a2l a1l a0l];
%Use the roots solver to solve the cubic equation for Z
Ztemp = roots(polyCol);
%display(Ztemp);
Zl = [];
%determine indices for roots are real.
index = find(imag(Ztemp)== 0);
%collect the real values of Z.
Zl = real(Ztemp(index));
Zlenght = length(Zl);
for i=1:Zlenght
   if Zl(i)<0
       Zl(i)=999;
   end
end
Zl = min(Zl);
% display(Zl)
if Zl == 999
    display('no liquid phase');
   
end

   compFugCoIl = exp(b/b_Mixl*(Zl-1)- ... 
                ones(1,NCOMP)*log(Zl-Bl)+...
                Al/(sqrt(8)*Bl) *((2*Psil)/ac_Mixl-b/b_Mixl).*log((Zl+(1-sqrt(2))*Bl) / (Zl+(1+sqrt(2))*Bl)));
    compFugIl = Pl* xi .* compFugCoIl;

%%%%%%%%%%%%%%%%%%%% 
%       VAPOUR     %
%%%%%%%%%%%%%%%%%%%%

ac_Mixv=0;
for j = 1:NCOMP
    for i = 1:1:NCOMP
       ac_Mixv = ac_Mixv+ yi(i)*yi(j)* sqrt(ac_alpha(i)*ac_alpha(j))*(1-kijmat(i,j));
    end
end

Av = ac_Mixv*Pv/(R*T)^2; %Equation 7.21
b_Mixv = sum(yi .* b);
Bv = b_Mixv*Pv/(R*T); %Equation 7.22

Psiv=zeros(1,NCOMP)  ; %595
for i = 1:NCOMP
    Psiv(i)=0;
    for j = 1:1:NCOMP
       Psiv(i) = Psiv(i)+ yi(j)* sqrt(ac_alpha(i)*ac_alpha(j))*(1-kijmat(i,j));
    end
end


%Calculate coefficients for the cubic equation to solve
a2v = -(1 - Bv);
a1v = Av-3*Bv^2-2*Bv;
a0v = -(Bv*Av-Bv^2-Bv^3);
%Store the coefficients in an array
polyCov = [1 a2v a1v a0v];
%Use the roots solver to solve the cubic equation for Z
Ztemp = roots(polyCov);
%display(Ztemp);
Zv = [];
%determine indices for roots are real.
index = find(imag(Ztemp)== 0);
%collect the real values of Z.
Zv = real(Ztemp(index));
Zv = max(Zv);
%display(Zv)
if Zv == 0
    display('no vapour phase');
end

   compFugCoIv = exp(b/b_Mixv*(Zv-1)- ... 
                ones(1,NCOMP)*log(Zv-Bv)+...
                Av/(sqrt(8)*Bv) *((2*Psiv)/ac_Mixv-b/b_Mixv).*log((Zv+(1-sqrt(2))*Bv) / (Zv+(1+sqrt(2))*Bv)));

    compFugIv = Pv * yi .* compFugCoIv;

end


