% PhD Thesis: Gas-Condensate Flow Modelling for Shale Gas Reservoirs
% Author: Ismail Labed
% Robert Gordon University
% October 2016
% Title: 3D Pore Network Modeling of Gas-Condensaet Flow in Shale Matrix
%============================================================================
%%
clc;
clear all;
Edge = 1;
%% PVT
T =393 ; %K
CosTheta = 0.8; %Condensate Wettability
Rg= 8314; %J/kmol/K

%Convertors
psitoPa = 6894.757;% From psi to MPa
cptoPas =	1.00E-03;
m2tomDarcy = 1.013249966e+15;
m2tonDarcy = 1.013249966e+21;

Alpha0=1;
A=1;
B=1;

%CORE
Dim =8;
CONN_FACTOR =3;
SKELETON =3;
DimX=Dim*10E-9;

% v  s Rmax Rmin
Dist =[10 0.6 60 0];
Dist(:,1)=Dist(:,1)*1E-9; Dist(:,3)=Dist(:,3)*1E-9;Dist(:,4)=Dist(:,4)*1E-9;
[DistNum, DistParaSize] = size(Dist);
Tho = 10;
Tolerance = 1E-12;
Transp=1;

PrRange = [2 5 8 11 14 18 21 26]; 
PVTFile='RICH_CVD_CH3.txt';
SatRange = [1:11];Sat_Stpe=22/10; %Sat_Liq(Sat_Liq_Iter) = (Sat_Liq_Iter-1)*Sat_Stpe/100;

%% Reading PVT Table
GasProp = InputGasProp_Network('RICH_CVD_CH3.txt');
[N,M] = size (GasProp); %N=80;

figure(10), set(gcf,'Color','w')
subplot(2,3,1),plot(GasProp(:,1), GasProp(:,7),'LineStyle','-','LineWidth',1,'Color','r'); grid,,xlabel('Pressure (psi)') , ylabel('Liquid Drop-out (fraction)') , title('Liquid Drop-out (CVD)'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
subplot(2,3,2),plot(GasProp(:,1), GasProp(:,17),'LineStyle','-','LineWidth',1,'Color','r'); grid,,xlabel('Pressure (psi)') , ylabel('IFT (mN/m)') , title('IFT'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on'
subplot(2,3,3),[ax,p1,p2] = plotyy(GasProp(:,1), GasProp(:,7),GasProp(:,1), GasProp(:,17),'plot','plot','LineWidth',1);
legend ('Liquid Drop-out', 'IFT'), xlabel(ax(1),'Pressure (psi)') , ylabel(ax(1),'Liquid Drop-out (fraction)') ,ylabel(ax(2),'IFT (mN/m)'), xlim(ax(1), [0 6000]),, xlim(ax(2), [0 6000])
ax = gca, ax.XGrid = 'on';ax.YGrid = 'on';ax = gca, ax.XMinorGrid = 'on';ax.YMinorGrid = 'on'

subplot(2,3,4),plot(GasProp(:,1), GasProp(:,2),'LineStyle','-','LineWidth',1,'Color','r'); grid,,xlabel('Pressure (psi)') , ylabel('u') , title('Viscosity, \mu'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
subplot(2,3,5),plot(GasProp(:,1), GasProp(:,3),'LineStyle','-','LineWidth',1,'Color','r'); grid,,xlabel('Pressure (psi)') , ylabel('Z') , title('Z-Factor'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
subplot(2,3,6),plot(GasProp(:,1), GasProp(:,17),'LineStyle','-','LineWidth',1,'Color','r'); grid,,xlabel('Pressure (psi)') , ylabel('MWg') , title('Gas MW'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
%%
for EXP =1:DistNum %expirement of v and s 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
'Distribution', EXP
v=Dist(EXP,1);%7E-9; %v_val(EXP);
s=Dist(EXP,2);%0.8; %s_val(EXP);
dR_ANA = v/100;
Rmin= Dist(EXP,4);
Rmax= Dist(EXP,3);

if SKELETON == 3;
Phi = 0.08;
LfromPhi = sqrt( pi * v^2* exp(s^2)/Phi);
DimX =Dim* LfromPhi*2/0.92 ; % m 
DimL(EXP) = DimX;
MaxCONN=3*DimX/Dim;
end

for k=1:Dim
    for j=1:Dim
        for i=1:Dim
            XYZ((k-1)*Dim^2+(j-1)*Dim+i,1)=(i-1)* DimX/Dim;
            XYZ((k-1)*Dim^2+(j-1)*Dim+i,2)=(j-1)* DimX/Dim;
            XYZ((k-1)*Dim^2+(j-1)*Dim+i,3)=(k-1)* DimX/Dim;
            
        end
    end
end

%%
 display('Gnerating CONN')
CONN = ones(Dim^3,Dim^3);


for M =1:CONN_FACTOR 
CONN = CONN .* (randi(2,[Dim^3,Dim^3])-ones(Dim^3,Dim^3));
end
CONN(1:Dim^2,1:Dim^2) = zeros(Dim^2,Dim^2); %delete Inlet-Inlet connection
CONN(Dim^3-Dim^2+1:Dim^3,Dim^3-Dim^2+1:Dim^3) = zeros(Dim^2,Dim^2); % delete Outlet-Outlet connection
%% SKELETON
if SKELETON == 1; 
CONN = zeros(Dim^3,Dim^3);
for i=1:Dim^3-Dim^2
 CONN(i,i+Dim^2)=1;
end
end

if SKELETON == 3; 
CONN = zeros(Dim^3,Dim^3);
for i=1:Dim^2
CONN(i,i+Dim^2)=1;
end

for i=Dim^2+1:Dim^3-Dim^2
 CONN(i,i+Dim^2)=1;CONN(i,i+1)=1; CONN(i,i+Dim)=1;
end
end
%%
for i=1:Dim^3 % delete same node-node connection and make CONN sym
    for j = i:Dim^3
        CONN(i,i)=0;
        CONN(j,i)=CONN(i,j);
    end
end

% CONNECTED NODE= number of connection in each node

% Look for dead-end and delete connection
while prod(sum(CONN (:,Dim^2+1:Dim^3-Dim^2))-ones(1,Dim^3-2*Dim^2)) == 0
for i=Dim^2+1:Dim^3-Dim^2
        if sum(CONN(i,:))== 1
        CONN(i,:)=zeros(1,Dim^3);
        CONN(:,i)=zeros(Dim^3,1);
        end
end
end

 %ACTIVE NODES
 CONNECTED_NODE = sum(CONN)'/2;
 ACTIVE_NODES = sum(CONNECTED_NODE > 0)
 
% Delete nodes withour in-out connection
IN_CONN=zeros(1,Dim^2);OUT_CONN=zeros(1,Dim^2);
for i=Dim^2+1:Dim^3-Dim^2
IN_CONN(i)=(sum(CONN(i,1:Dim^2))>0);
end
for m=1:10
    for i=Dim^2+1:Dim^3-Dim^2
    for j=Dim^2+1:Dim^3-Dim^2
        if IN_CONN(i)==0
    IN_CONN(i)=IN_CONN(j) * CONN(i,j);
        end
    end
    end
end

for i=Dim^2+1:Dim^3-Dim^2
OUT_CONN(i)=(sum(CONN(i,Dim^3-Dim^2+1:Dim^3))>0);
end
for m=1:10
    for i=Dim^2+1:Dim^3-Dim^2
    for j=Dim^2+1:Dim^3-Dim^2
        if OUT_CONN(i)==0
    OUT_CONN(i)=IN_CONN(j) * CONN(i,j);
        end
    end
    end
end
IN_OUT_CONN = IN_CONN.*OUT_CONN;

for i=Dim^2+1:Dim^3-Dim^2
for j=1:Dim^3
CONN(i,j)=IN_OUT_CONN(i)*CONN(i,j);
CONN(j,i)=CONN(i,j);
end
end

% ACTIVE NODES
CONNECTED_NODE = sum(CONN)'/2;
ACTIVE_NODES = sum(CONNECTED_NODE > 0)
PDim = sum(IN_OUT_CONN(Dim^2+1:Dim^3-Dim^2))
%% L MATRIX
%%%%%%%%%%%%%%%
 display('Gnerating L')

for i=1:Dim^3 
for j = 1:Dim^3
COORD1 = XYZ(i,1:3);
COORD2 = XYZ(j,1:3);
L(i,j)=  (sum((COORD1-COORD2).^2))^0.5  *CONN(i,j) ; %m
end
end

% Eleminiate Connection with L > MaxCONN
MaxCONN=3*DimX/Dim;
for i=1:Dim^3
    for j = 1:Dim^3
if L(i,j) > MaxCONN
    CONN(i,j)=0;CONN(j,i)=0;
    L(i,j)=0;L(j,i)=0;
end 
    end
end
%% R Matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 display('Gnerating R')
R_Vector_Size = sum(sum(CONN))/2;
R_Vector = lognrnd(log(v),s,R_Vector_Size,1);

for i=1:R_Vector_Size
%  if R  > Rmax || R_Vector(i) < Rmin
    if R_Vector(i) > Rmax || R_Vector(i) < Rmin
        R_Vector(i)=Rmin + rand * (Rmax - Rmin);
    end
end

RminANA = min(R_Vector); RmaxANA=max(R_Vector); 


R_Index=1;

for i=1:Dim^3
    for j = i:Dim^3
        if CONN(i,j)~=0 
        R(i,j)=R_Vector(R_Index,1);
    R_Index=R_Index + 1;
        else R(i,j)=0;
        end
    end
end

for i=1:Dim^3
    for j =1: i
        R(i,i)=0;
        R(i,j)=R(j,i);
    end
end
% Fit Function
R_Fit = (0.01:0.01:max(R_Vector*1E9))*1E-9;
PDF_Fit = lognpdf(R_Fit*1E9,log(v*1E9),s);
display('LogNorFit')
% R_POREs
R_Pores = max(R)*0; %lognrnd(log(v_pore),s_pore,R_Vector_Size,1); %ones(Dim^3,1)*0E-9; %m

%% Saturation & Pcap
'Genrating S=f(Rming) and Pcap'
PressIndex = [3 7 13 22]; PcapNum=4;
[ R_SORTED, Sat_Rming,Pca_PM] = SatToRmin4_Pores( Dim, R,DimX/Dim, R_Pores, GasProp, PressIndex, PcapNum ,CosTheta);

%%
figure(20),set(gcf,'Color','w')
subplot(2,3,1),histogram(R_Vector*1E9,100), title('Shape of Ditribution'), grid, xlabel('Pore Radius, R  (nm)'), ylabel('Frequency'), title(' Pore Radius Histogram'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
subplot(2,3,2),[ax,p1,p2] = plotyy(R_Vector*1E9,100,R_Fit*1E9,PDF_Fit,'histogram','plot','LineWidth',1);
legend ('Radius Ditribution', 'Log-normal PDF'), xlabel(ax(1),'Pore Radius, R  (nm)') , ylabel(ax(1),'Frequency') ,ylabel(ax(2),'PDF')
subplot(2,3,4), plot(Sat_Rming(1:1000,2)*1E9,Sat_Rming(1:1000,1), '-', 'LineWidth',2), grid, ylabel('Condensate Saturation, S_c (fraction)'), xlabel('R_{g,min} (m)'),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
%% for PcapGraph=1:PcapNum
% Pcap_Pr_Value(PcapGraph) = GasProp(PressIndex(PcapGraph),1);
%  subplot(2,3,5), plot(Pca_PM(1:1000,1),Pca_PM(1:1000,PcapGraph+1), '-', 'LineWidth',2), hold all
%  end
%  grid, ylabel('Capillary Pressure, P_c (psi)'), xlabel(' Condensate Saturation, S_c (fraction)'), legend(num2str(Pcap_Pr_Value')),ax = gca; ax.XMinorGrid = 'on';ax.YMinorGrid = 'on';
% [ A ] =NetworkPlot2( Dim,CONN,XYZ, R,R_Pores,L, Transp, DimX, Rmax)
%  'pause', pause,
%% %% Volumes
% %%%%%%%%%%%%
VOL = pi *L.*R.^2 ; % Tube volume m3
VOL_Pores = 4/3 *pi * sum(R_Pores.^3); % Pore volume m3
PVOL_TOT = sum ( sum(VOL)) + VOL_Pores; % Total pore volume m3
PVOL_Frac = VOL/PVOL_TOT; % Frction Tube volume
BULK_VOL = DimX^3; %m3
PORO(EXP) = PVOL_TOT/BULK_VOL
% 'pause', pause
%%
INIT_CONN = CONN;
INIT_R = R;
   
    QPLUS=zeros(100,N);
    plothist = 1; 

for Press_Iter= PrRange 
%%  Press_Iter
display('__________________________________________________________________________')
Pinlet = (GasProp(Press_Iter,1)+5)*psitoPa; %psi
Poutlet = (Pinlet - 5*psitoPa) ; %psi

IFT = GasProp(Press_Iter,17);

for Sat_Liq_Iter = SatRange 
    Sat_Liq(Sat_Liq_Iter) = (Sat_Liq_Iter-1)*Sat_Stpe/100;
clear Pmid P M dM_dPj J Const NOT_CONN;

Rming = Find_Rmin( Sat_Liq(Sat_Liq_Iter), Sat_Rming );

%% Phase Distribution
CONN =INIT_CONN;
CONN_Gas =INIT_CONN;
CONN_Liq =INIT_CONN;
R =INIT_R;

for i=1:Dim^3
for j = 1:Dim^3
    if INIT_CONN(i,j)==1
        if R(i,j)> Rming % Gas Tube 
        R_Gas(i,j) = R(i,j);
        R_Liq(i,j)=0;R_Liq(j,i)=0;
        CONN_Liq(i,j)=0;CONN_Liq(j,i)=0;
        else % Liquid Tube
        R_Liq(i,j) = R(i,j);
        R_Gas(i,j)=0;R_Gas(j,i)=0;
        CONN_Gas(i,j)=0;CONN_Gas(j,i)=0;
        end
    else
    R_Liq(i,j)=0;R_Liq(j,i)=0;
    R_Gas(i,j)=0;R_Gas(j,i)=0;
    end
end
end

%% Delete nodes withour in-out connection
[ CONN_Gas, NOT_CONN_Gas,IN_OUT_CONN_Gas ] = CONN_Fun( Dim, INIT_CONN, CONN_Gas );
[ CONN_DG, NOT_CONN_DG,IN_OUT_CONN_DG ] = CONN_Fun( Dim, INIT_CONN, CONN );
[ CONN_Liq, NOT_CONN_Liq,IN_OUT_CONN_Liq ] = CONN_Fun( Dim, INIT_CONN, CONN_Liq );
%% POIS_FACT = pi/8 * R^4/L
[ POIS_FACT ] = POIS_FACT_Fun( R,L, Dim );
%% PLOT
% figure(655)
% [ A ] =NetworkPlot3( Dim,CONN_Gas,CONN_Liq,XYZ, R_Gas, R_Liq,R_Pores,L, 0.05,0.5, DimX, Rmax)
%% Capillary Pressure for Condensate Trapping
%%Capillary pressure in liquid tubes
% CH3_Network_Liquid_CapPr.m
%% MODEL SETTING
[ PDim_Gas, P_Gas, INDEX_Gas, Pmid_Gas ] = P_Vector( Dim,CONN_Gas, IN_OUT_CONN_Gas,Pinlet, Poutlet);
[ PDim_DG, P_DG, INDEX_DG, Pmid_DG ] = P_Vector( Dim,INIT_CONN, IN_OUT_CONN_DG,Pinlet, Poutlet);
[ PDim_Liq, P_Liq, INDEX_Liq, Pmid_Liq ] = P_Vector( Dim,CONN_Liq, IN_OUT_CONN_Liq,Pinlet, Poutlet);
%% Flow Solver
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Int_Perm
[ Perm_Int, QRate,u_avg, Iter, Rel_ERR] = ...
        Solver( GasProp,Dim,INIT_CONN,R, POIS_FACT, psitoPa, T, Rg,Tolerance, 0, PDim_DG,INDEX_DG, P_DG, Pmid_DG, DimX ,Tho,Pinlet,Poutlet);    
%% Gas Perm
if PDim_Gas ==0
Perm_Gas_GC_Darcy=0, Perm_Gas_GC_Kn=0
else 
    % KN = 0  GC Darcy
    [ Perm_Gas_GC_Darcy, QRate,u_avg, Iter, Rel_ERR] = ...
        Solver( GasProp,Dim,CONN_Gas,R_Gas, POIS_FACT, psitoPa, T, Rg,Tolerance, 0, PDim_Gas,INDEX_Gas, P_Gas, Pmid_Gas, DimX ,Tho,Pinlet,Poutlet,Alpha0, A, B);    
    % KN = 1  GC Knudsen
    [ Perm_Gas_GC_Kn, QRate,u_avg, Iter, Rel_ERR] = ...
        Solver( GasProp,Dim,CONN_Gas,R_Gas, POIS_FACT, psitoPa, T, Rg,Tolerance, 1, PDim_Gas,INDEX_Gas, P_Gas, Pmid_Gas, DimX ,Tho,Pinlet,Poutlet,Alpha0, A, B);    
end
    
    % KN = 2  DRY GAS Knudsen
    [ Perm_Gas_DG, QRate,u_avg, Iter, Rel_ERR] = ...
        Solver( GasProp,Dim,INIT_CONN,R, POIS_FACT, psitoPa, T, Rg,Tolerance, 2, PDim_DG,INDEX_DG, P_DG, Pmid_DG, DimX ,Tho,Pinlet,Poutlet,Alpha0, A, B);    

%%
if PDim_Liq ==0
Perm_Liq =0
else
[ Perm_Liq, QRate,u_avg, Iter, Rel_ERR] = Solver( GasProp,Dim,CONN_Liq,R_Liq, POIS_FACT, psitoPa, T, Rg,Tolerance, 0, PDim_Liq,INDEX_Liq, P_Liq, Pmid_Liq, DimX ,Tho,Pinlet,Poutlet);
end
%% Results;[
% 1.Pressure    
% 2.Saturation
% 3.Perm Darcy
% 4.Perm Knudsen
% 5.Perm Dry gas knudsen
% 6.Perm Liquid
% 10.REL_PERM Darcy 
% 11.REL_PERM Knudsen
% 12.REL_PERM Liq
Results(Press_Iter,Sat_Liq_Iter,1) = GasProp(Press_Iter,1); %Pressure
Results(Press_Iter,Sat_Liq_Iter,2) = Sat_Liq(Sat_Liq_Iter); %Saturation
Results(Press_Iter,Sat_Liq_Iter,3) = Perm_Gas_GC_Darcy; % PERM Gas Darcy
Results(Press_Iter,Sat_Liq_Iter,4) = Perm_Gas_GC_Kn; % PERM Gas Knudsen
Results(Press_Iter,Sat_Liq_Iter,5) = Perm_Gas_DG; % PERM Gas Dry gas knudsen
Results(Press_Iter,Sat_Liq_Iter,6) = Perm_Liq; % PERM Liq
Results(Press_Iter,Sat_Liq_Iter,7) = Perm_Int;

E_SIM(Press_Iter,Sat_Liq_Iter)= Perm_Gas_GC_Kn/Perm_Gas_GC_Darcy;
E_SIM_DRY(Press_Iter,Sat_Liq_Iter)= Perm_Gas_DG/Perm_Int;
E_REL (Press_Iter,Sat_Liq_Iter)=E_SIM(Press_Iter,Sat_Liq_Iter)/E_SIM_DRY(Press_Iter,Sat_Liq_Iter);

REL_Gas(Press_Iter,Sat_Liq_Iter) = Perm_Gas_GC_Darcy/Perm_Int; % REL_PERM Darcy
REL_Liq(Press_Iter,Sat_Liq_Iter) = Perm_Liq/Perm_Int; % REL_PERM Darcy

%% Sat Results
if KN==0
Results(Sat_Liq_Iter,20) = Rming;
Results(Sat_Liq_Iter,21) = u_avg; 
Results(Sat_Liq_Iter,22) = QRate; 
end

%% Log
msg = sprintf('%s %1.0f %s %1.0f %s %1.3f %s %1.0f %s %1.2f %s %1.0f %s %1.0f %s %1.0f %s %1.0f %s %1.4f %s %1.0f',...
              'EXP=', EXP,'v= ',v*1E9,'s= ',s, 'Pr= ', GasProp(Press_Iter,1),'Sc=',Sat_Liq(Sat_Liq_Iter), 'PDim= ',PDim, 'Perm_Int=', Perm_Int, 'Krg_Darcy=',Perm_Gas_GC_Darcy, 'Krc=',Perm_Liq, 'PORO=',PORO(EXP),'Rming',Rming*1E9); disp(msg);

%% Class Contribution
if KN_EXP ==3
figure(1100)
[ QClass,RClass,RClassValue ] = QContr( R,Q, Dim, Rmin , Rmax,QRate );
stairs(RClassValue*1E9, QClass,'LineStyle','-'), hold all
xlabel('Pore Radius(nm)')
ylabel('Contribution to total gas rate')
end

end % Sat_Liq
%for Press_Iter= PrRange
figure(90)
subplot(2,3,1), plot(Results(Press_Iter,:,2),REL_Gas(Press_Iter,:) ,'o',...
                     Results(Press_Iter,:,2),REL_Liq(Press_Iter,:),'+'),hold all
title('Darcy REL PERM'), legend('Gas', 'Condensate'), xlim([0 1 ])
xlabel('Condensate Saturation S_g'),ylabel('Relative Permeability')

subplot(2,3,2), plot(Results(Press_Iter,:,2),E_REL (Press_Iter,:),'-+', 'Markersize', 4,'LineWidth',1),hold all
grid, grid minor
xlabel('Condensate Saturation S_c'),ylabel('Relative Correction Factor, \xi_{Rel}')
legend( ['Pr = 14.7'],['Pr = 50'],['Pr = 100'],['Pr = 200'],['Pr = 500'],['Pr = 1000'],['Pr = 2200'])

end % Press_Iter
%%
figure (50),set(gcf,'Color','w')
subplot(2,3,1), plot(Results(:,1),Results(:,2),'-o'), title(' Liq drop out')
subplot(2,3,2), plot(Results(:,1),Results(:,3),'-+',Results(:,1),Results(:,4),'-s',Results(:,1),Results(:,5),'-o'), title('Gas Permeability '), hold all
legend('Darcy', 'Knudsen', 'Knudsen Dry gas')
xlabel('Pressure (psi)')
ylabel('Permeability (nDarcy)')

subplot(2,3,2), plot(Results(:,2,1),E_REL (:,2),'-+', 'Markersize', 4,'LineWidth',1),hold all
legend('\xi_{GC}','\xi_{DG}'), title('E Factor '),xlabel('Pressure (psi)'),ylabel('Correction Factor, \xi')

subplot(2,3,6), plot(Results(:,1),REL_E(EXP,:),'-+', 'Markersize',4),hold all

 end  %EXP


