% PhD Thesis: Gas-Condensate Flow Modelling for Shale Gas Reservoirs
% Author: Ismail Labed
% Robert Gordon University
% October 2016
% Title: Phase Behaviour Modelling in Pore Network_Dew Point Estimation
%============================================================================
% LIBRARY

% Mol Weight| Crit Pres| Crit Temp| Acentric Factor| Parachors| V Crit (Visc)
% 	              (MPa)	      (K)			                       (m3 /kg-mole)
%____________________________________
%	1	C1	|	21	C17	|	41	C37	|
%	2	C2	|	22	C18	|	42	C38	|
%	3	C3	|	23	C19	|	43	C39	|
%	4	NC4	|	24	C20	|	44	C40	|
%	5	C4	|	25	C21	|	45	C41	|
%	6	IC4	|	26	C22	|	46	C42	|
%	7	NC5	|	27	C23	|	47	C43	|
%	8	C5	|	28	C24	|	48	C44	|
%	9	IC5	|	29	C25	|	49	C45	|
%	10	C6	|	30	C26	|	50	H2	|
%	11	C7	|	31	C27	|	51	N2	|
%	12	C8	|	32	C28	|	52	CO	|
%	13	C9	|	33	C29	|	53	CO2	|
%	14	C10	|	34	C30	|	54	H2S	|
%	15	C11	|	35	C31	|	55	H2O	|
%	16	C12	|	36	C32	|			|
%	17	C13	|	37	C33	|			|
%	18	C14	|	38	C34	|			|
%	19	C15	|	39	C35	|			|
%	20	C16	|	40	C36	|			|
%%____________________________________	

clc;
clear all;

[Library, BIC ] = ReadLib(  );
%% INPUT Composition

%% Rich Cond
Composition = [1    5   11  14  18];
zi =          [78   12  7.2 1.4 1.4]/100;
% Composition = [1    14 ];
% zi =          [95   5 ]/100;
%% Lean Cond
% Composition = [1        5   11  10  18];
% zi =          [83.2     13  3.1 0.4 0.3]/100;


PMIN = 500/145.037738007;
PMAX = 1500/145.037738007;
NNP = (PMAX- PMIN)*145.037738007;

TMIN = 380;%K Reservoir Temp
TMAX = 485;
NNT = (TMAX- TMIN)/5;

CosTheta = 0.8; %Condensate Wettability

v=10E-9; %m Pore Size Median
s=0.3;  % Pore Standard Deviataion

NSynth =1; % number of synth tubes
Rmin= 1E-9;
Rmax=50E-9;
dR = 1E-10;
%% Data Validation
if ne(sum(zi),1), error ('Sum of Composition Fractions unequal 1'), end
%% Data Import
[N,M]=size (Composition);
NCOMP=M;
for i =1:NCOMP
MW(i) = Library(Composition(i),1);  
Pc(i) = Library(Composition(i),2); %(MPa)	
Tc(i) = Library(Composition(i),3); %K
w(i)  = Library(Composition(i),4); % ACENTRIC FACTOR
Parachor(i) =Library(Composition(i),5);
Vc(i) = Library(Composition(i),6); %  (m3 /kg-mole)
Vc(i) = Vc(i)/1E-3; %  (cm3 /g-mole)
end

for i=1:NCOMP
    for j=1:NCOMP
        kijmat(i,j) = BIC(Composition(i),Composition(j));
end
end
%%  Devide Matrix to n Tube
%RSynth(1,i): R SYNTH
%RSynth(2,i)  f(R)
%RSynth(3,:): fraction of volume by radius
%RSynth(4,i) cumulatuive RSynth(3,:)
RSynth = RSynth_Fun( v,s,NSynth,Rmin,Rmax, dR )';

%% BULK Calculation

for NT=1:NNT
T(NT) = TMAX - (TMAX - TMIN)/NNT * (NT - 1)  ;
[ DP(NT), IFT(NT)] = DEWPOINT(NCOMP,T(NT), zi, PMAX,Tc, Pc, w, Parachor, MW,Vc,kijmat,1);
end


%%
R_Class=[3 5 10 50 100 ];
for i= 1:5
for NT=1:NNT
[ CAP_DP(NT,i), CAP_IFT(NT,i)] = CAP_DEWPOINT(NCOMP,T(NT), zi, DP(NT),Tc, Pc, w, Parachor, MW,Vc,kijmat, CosTheta, R_Class(i),IFT(NT),0);
end
end

%% plot
subplot(2,2,1), plot(T,DP*145.037738007,'LineWidth', 1), hold all
subplot(2,2,3), plot(T,IFT*145.037738007,'LineWidth', 1), hold all
for i= 1:5
subplot(2,2,1), plot(T,CAP_DP(:,i)*145.037738007, 'LineStyle', '-.','LineWidth', 1), hold all
subplot(2,2,2), plot(T,(CAP_DP(:,i)-DP')*145.037738007, 'LineStyle', '-.','LineWidth', 1), hold all
subplot(2,2,3), plot(T,CAP_IFT(:,i)*145.037738007, 'LineStyle', '-.','LineWidth', 1), hold all
end
subplot(2,2,1),legend('Bulk', '3nm', '5nm', '10nm', '50nm', '100nm'), grid on,grid minor, ylabel ('Pressure (psi)'),xlabel ('Temperature (K)')
subplot(2,2,2),legend('3nm', '5nm', '10nm', '50nm', '100nm'), grid on, grid minor, ylabel ('Dew Point Deviation (psi)'),xlabel ('Temperature (K)')
subplot(2,2,3),legend('Bulk', '3nm', '5nm', '10nm', '50nm', '100nm'), grid on, grid minor, xlabel ('Temperature (K)'), ylabel ('IFT (mN/m)')


