% PhD Thesis: Gas-Condensate Flow Modelling for Shale Gas Reservoirs
% Author: Ismail Labed
% Robert Gordon University
% October 2016
% Title: Phase Behaviour Modelling in Pore Network_Discretised Pore Space Model (DPSM)
%============================================================================
% LIBRARY

% Mol Weight| Crit Pres| Crit Temp| Acentric Factor| Parachors| V Crit (Visc)
% 	              (MPa)	      (K)			                       (m3 /kg-mole)
%____________________________________
%	1	C1	|	21	C17	|	41	C37	|
%	2	C2	|	22	C18	|	42	C38	|
%	3	C3	|	23	C19	|	43	C39	|
%	4	NC4	|	24	C20	|	44	C40	|
%	5	C4	|	25	C21	|	45	C41	|
%	6	IC4	|	26	C22	|	46	C42	|
%	7	NC5	|	27	C23	|	47	C43	|
%	8	C5	|	28	C24	|	48	C44	|
%	9	IC5	|	29	C25	|	49	C45	|
%	10	C6	|	30	C26	|	50	H2	|
%	11	C7	|	31	C27	|	51	N2	|
%	12	C8	|	32	C28	|	52	CO	|
%	13	C9	|	33	C29	|	53	CO2	|
%	14	C10	|	34	C30	|	54	H2S	|
%	15	C11	|	35	C31	|	55	H2O	|
%	16	C12	|	36	C32	|			|
%	17	C13	|	37	C33	|			|
%	18	C14	|	38	C34	|			|
%	19	C15	|	39	C35	|			|
%	20	C16	|	40	C36	|			|
%%____________________________________	
tic;
clc;
clear all;

[Library, BIC ] = ReadLib(  );
%% INPUT Composition

%% Rich Cond
% Composition = [1    2   3  5  8  10 16];
% zi =                [70   9   6  6  2  1  6]/100;

%% Lean Cond
Composition = [1    5   11  14  18];
zi =          [80   12  7.2 0.4 0.4]/100;

PSTEP= 50/145.0377;
PMIN =500/145.0377;
PMAX = 1050/145.0377;
NNP = round(( PMAX-PMIN)/PSTEP);


T= 392; %K Reservoir Temp

CosTheta = 0.8; %Condensate Wettability
%%
v=10; %m Pore Size Median
s=0.6;  % Pore Standard Deviataion
Rmin= 3;
Rmax=70;
Rnum=15;
Rmean = v*exp(s^2/2)*2/(erf( log(Rmax/v) /(sqrt(2)*s)) - (erf( log(Rmin/v) /(sqrt(2)*s))));
%%  Devide Matrix to n Tube

R=[0:0.05:2.3];
R=10.^(R);
PDF =lognpdf(R, log(v), s) *2/(erf( log(Rmax/v) /(sqrt(2)*s)) - (erf( log(Rmin/v) /(sqrt(2)*s))));
CDF =logncdf(R, log(v), s)*2 /(erf( log(Rmax/v)/(sqrt(2)*s)) - (erf( log(Rmin/v) /(sqrt(2)*s))));
% semilogx(R*1E9,PDF,R*1E9,CDF)
R_Class=[log10(Rmin):        ( log10(Rmax)-log10(Rmin) ) /(Rnum-1)     :log10(Rmax)];
R_Class=10.^(R_Class);
CDF_Class =logncdf(R_Class, log( v), s)*2 /(erf( log(Rmax/v) /(sqrt(2)*s)) - (erf( log(Rmin/v) /(sqrt(2)*s))));
Inc_PSpase(1)=CDF_Class(1);

for i=2:Rnum
Inc_PSpase(i)=CDF_Class(i)-CDF_Class(i-1);
end
Inc_PSpase= Inc_PSpase* pi .* R_Class.^2;
Inc_PSpase= Inc_PSpase/sum(Inc_PSpase);

figure(300), set(gcf,'Color','w')
% plotyy(R,PDF,R,CDF,'semilogx','semilogx','LineWidth',1);
subplot(2,2,1),plotyy(R,PDF,R_Class,Inc_PSpase,'semilogx','semilogx');
ax = gca, ax.XGrid = 'on';ax.YGrid = 'on';ax = gca, ax.XMinorGrid = 'on';ax.YMinorGrid = 'on'
legend ('PDF','Incremental Pore Space'), xlabel(ax(1),'Pore Radius (nm)') 
ylabel(ax(1),'PDF') 

subplot(2,2,2),stairs(cumsum(Inc_PSpase), R_Class) , hold on
subplot(2,2,2),stairs(cumsum(Inc_PSpase), - R_Class) 
 xlabel('Incremental Pore Space')
ylabel('Pore Radius (nm)')
%% Data Validation
if ne(sum(zi),1)
    error ('Sum of Composition Fractions unequal 1')
end

%% Data Import
[N,M]=size (Composition);
NCOMP=M;

for i =1:NCOMP
MW(i) = Library(Composition(i),1);  
Pc(i) = Library(Composition(i),2); %(MPa)	
Tc(i) = Library(Composition(i),3); %K
w(i)  = Library(Composition(i),4); % ACENTRIC FACTOR
Parachor(i) =Library(Composition(i),5);
Vc(i) = Library(Composition(i),6); %  (m3 /kg-mole)
Vc(i) = Vc(i)/1E-3; %  (cm3 /g-mole)
end
for i=1:NCOMP
    for j=1:NCOMP
        kijmat(i,j) = BIC(Composition(i),Composition(j));
end
end
%% Pcap

%% BULK Calculation
'BULK Calculation'
figure(200)
[ DP, IFT_DP] = DEWPOINT(NCOMP,T, zi, PMAX,Tc, Pc, w, Parachor, MW,Vc,kijmat,1); PMAX= DP; NNP = round(( PMAX-PMIN)/PSTEP);
[ BULKliqsat, IFT_BULK, BULKViscVisc, GasComp,LiqComp] = BULK_CVD(NCOMP,T, zi, PMIN, PMAX, NNP,Tc, Pc, w, Parachor, MW, Vc, kijmat,1);
% [ BULKliqsat, IFT_BULK, BULKViscVisc, GasComp,LiqComp] = BULK_CCE(NCOMP,T, zi, PMIN, PMAX, NNP,Tc, Pc, w, Parachor, MW, Vc, kijmat,1);
plot(BULKliqsat(:, 1)*145.0377,BULKliqsat(:,2))
% 'pause',pause
%% 
'CAP Calculation'
CAPliqsatMatrix=zeros(NNP+1,1);
for i=1:Rnum
 iteration =1
errIFT =1;
IFT = IFT_BULK;
[ CAP_DP] = CAP_DEWPOINT(NCOMP,T, zi, DP,Tc, Pc, w, Parachor, MW,Vc,kijmat, CosTheta, R_Class(i),IFT_DP,0); CAP_PMAX= CAP_DP;

while errIFT > 1e-3
    IFT_OLD = IFT;
[ CAPliqsat, IFT, CAPVisc, CAPGasComp] =  CAP_CVD(NCOMP,T, zi, PMIN, CAP_PMAX, NNP,Tc, Pc, w, Parachor, MW, Vc, kijmat, IFT_OLD, CosTheta, R_Class(i),1);

errIFT= sum ( ((IFT- IFT_OLD)./IFT_OLD).^2) ;

iteration =iteration +1;

if iteration >20
    errIFT = 0;
    'Aborted'
end

end
CAPliqsatTube(:,i)=   CAPliqsat(:,2);



end
CAPliqsatPM= CAPliqsatTube * Inc_PSpase';

subplot(2,2,4)
plot(BULKliqsat(:, 1)*145.0377,BULKliqsat(:,2),BULKliqsat(:, 1)*145.0377,CAPliqsatPM,'LineStyle','-','LineWidth',1), hold all
xlabel ('Pressure, psi'), ylabel('Condensate Satuartion, Fraction')
ylim([0 max(CAPliqsat(:,2))+0.05 ])
grid

subplot(2,2,1)
plot(BULKliqsat(:, 1)*145.0377,BULKliqsat(:,2),'r','LineStyle','-','LineWidth',1), hold all
for i=1:Rnum
subplot(2,2,1)
plot(CAPliqsat(:, 1)*145.0377,CAPliqsatTube(:,i),'LineStyle','-.','LineWidth',1), hold all
end
xlabel ('Pressure, psi'), ylabel('Condensate Satuartion, Fraction')
ylim([0 max(CAPliqsat(:,2))+0.05 ])
grid
legend('Bulk', '3nm', '5nm', '10nm', '20nm', '30nm' )
%% Rmean

 iteration =1;
errIFT =1;
IFT = IFT_BULK;
[ CAP_DP] = CAP_DEWPOINT(NCOMP,T, zi, DP,Tc, Pc, w, Parachor, MW,Vc,kijmat, CosTheta, Rmean,IFT_DP,0); CAP_PMAX= CAP_DP;

while errIFT > 1e-12
IFT_OLD = IFT;
[ CAPliqsatRmean, IFT, CAPVisc, CAPGasComp] =  CAP_CVD(NCOMP,T, zi, PMIN, CAP_PMAX, NNP,Tc, Pc, w, Parachor, MW, Vc, kijmat, IFT_OLD, CosTheta, Rmean,0);
errIFT= sum ( ((IFT- IFT_OLD)./IFT_OLD).^2) ;
iteration =iteration +1;

if iteration >20
    errIFT = 0;
    'Aborted'
end

end
subplot(2,2,4)
plot(BULKliqsat(:, 1)*145.0377,CAPliqsatRmean(:,2),'LineStyle','-','LineWidth',1), hold all
xlabel ('Pressure, psi'), ylabel('Condensate Satuartion, Fraction')
ylim([0 max(CAPliqsat(:,2))+0.05 ])
grid
legend('Bulk', 'DPSM', 'R_{avg}')
toc;